#ifndef _g_kde_h
#define _g_kde_h


#include <qevent.h>
#include <qframe.h>
#include <qkeycode.h>
#include <qlayout.h>
#include <qobject.h>
#include <qscrbar.h>
#include <qwidget.h>
#include <qmultilineedit.h>


#include "sysdep.h"
#include "console.h"
#include "gui.h"

#include <kmainwindow.h>

#define MAX_PIPES 4

class PipeReader;

typedef struct {
    int used;
    int id;
    int fd;
    int pid;
    int stopped;
    EModel *notify;
    //XtInputId input;
    PipeReader *reader;
} GPipe;

class PipeReader : public QObject {
    Q_OBJECT
public slots:
    void doRead();
public:
    GPipe *pipe;
};

#define sfFocus   1

class QEText: public QWidget {
    Q_OBJECT
public:
    GViewPeer   *view;
    
    QEText(GViewPeer *peer, QWidget *parent = 0, const char *name = 0);
    virtual ~QEText();

    virtual void handleKeyPressEvent(QKeyEvent *qe);
    void ActiveEvent(TEvent &Event);
protected:
    void handleMouse(QMouseEvent *qe);
        
    virtual void resizeEvent(QResizeEvent *qe);
    virtual void paintEvent(QPaintEvent *qe);
    virtual void mousePressEvent(QMouseEvent *qe);
    virtual void mouseMoveEvent(QMouseEvent *qe);
    virtual void mouseReleaseEvent(QMouseEvent *qe);
//    virtual void keyPressEvent(QKeyEvent *qe);
};

class QEView: public QFrame {
    Q_OBJECT
public:
    GViewPeer   *view;
    QEText      *text;
    QScrollBar  *horz;
    QScrollBar  *vert;
    
    QEView(GViewPeer *peer, QWidget *parent = 0, const char *name = 0);
    virtual ~QEView();

    void ActiveEvent(TEvent &Event);

protected:
//    virtual void keyPressEvent(QKeyEvent *qe);
//    virtual void resizeEvent(QResizeEvent *qe);

protected slots:
    void sbHmoveLeft();
    void sbHmoveRight();
    void sbHpageLeft();
    void sbHpageRight();
    void sbHmoveTo(int pos);

    void sbVmoveUp();
    void sbVmoveDown();
    void sbVpageUp();
    void sbVpageDown();
    void sbVmoveTo(int pos);
};

class QEFrame : public KMainWindow {
    Q_OBJECT
public:
    //KMenuBar    *menubar;
    KToolBar    *toolbar;
    virtual ~QEFrame(){};
    QWidget *innerFrame;
    QEFrame(GFramePeer *peer, QWidget *parent = 0, const char *name = 0);

    void CreateMenuBar(int Id);
    QPopupMenu *CreatePopup(QWidget *parent, int Id, int do_connect);
    QList <QEView> viewList;
    QList <QWidget> popupList;
    QList <QWidget> oldPopupList;
    QVBoxLayout *layout;
    void tileViews();
    void createToolBar();
protected:
//    virtual void resizeEvent(QResizeEvent *qe);
    virtual void closeEvent(QCloseEvent *qe);
    virtual void keyPressEvent(QKeyEvent *qe);
public slots:
    void selectedMain(int id);
    void timerDone();
private:
    GFramePeer  *frame;
};

class GViewPeer {
public:
    QEView *qView;
    GC GCs[256];
    
    GView *View;
//    int wX, wY;
    int wW, wH, wState, wRefresh;
    int cX, cY, cVisible, cStart, cEnd;
    int sbVstart, sbVamount, sbVtotal;
    int sbHstart, sbHamount, sbHtotal;
    int VertPos, HorzPos;
    unsigned char *ScreenBuffer;
    
    GViewPeer(GView *view, int XSize, int YSize);
    ~GViewPeer();
    
    int AllocBuffer();
    void DrawCursor(/*QPainter *painter, */int Show);
    void UpdateWindow(int xx, int yy, int ww, int hh);

    int ConPutBox(int X, int Y, int W, int H, PCell Cell);
    int ConGetBox(int X, int Y, int W, int H, PCell Cell);
    int ConPutLine(int X, int Y, int W, int H, PCell Cell);
    int ConSetBox(int X, int Y, int W, int H, TCell Cell);
    int ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count);

    int ConSetSize(int X, int Y);
    int ConQuerySize(int *X, int *Y);
    
    int ConSetCursorPos(int X, int Y);
    int ConQueryCursorPos(int *X, int *Y);
    int ConShowCursor();
    int ConHideCursor();
    int ConCursorVisible();
    int ConSetCursorSize(int Start, int End);
    
    int QuerySbVPos();
    int SetSbVPos(int Start, int Amount, int Total);
    int SetSbHPos(int Start, int Amount, int Total);
    int ExpandHeight(int DeltaY);

    int UpdateCursor();
    int PMShowCursor();
    int PMHideCursor();
    int PMSetCursorPos();
};      

class GFramePeer {
public:
    GFrame *Frame;
    QEFrame *qFrame;
    
    GFramePeer(GFrame *aFrame, int Width, int Height);
    ~GFramePeer();
    
    int ConSetTitle(char *Title, char *STitle);
    int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen);
    
    int ConSetSize(int X, int Y);
    int ConQuerySize(int *X, int *Y);
    void MapFrame();
    void ShowFrame();
};

typedef struct _qEvent {
    TEvent event;
    struct _qEvent *next;
} qEvent;


static struct {
    unsigned int q_code;
    TKeyCode keyCode;
} key_table[] = {
{ Qt::Key_Escape,     kbEsc },
{ Qt::Key_Tab,        kbTab },
{ Qt::Key_Backtab,    kbTab | kfShift },
{ Qt::Key_Backspace,  kbBackSp },
{ Qt::Key_Return,     kbEnter },
{ Qt::Key_Enter,      kbEnter },
{ Qt::Key_Insert,     kbIns },
{ Qt::Key_Delete,     kbDel },
{ Qt::Key_Pause,      kbPause },
{ Qt::Key_Print,      kbPrtScr },
{ Qt::Key_SysReq,     kbSysReq },
{ Qt::Key_Home,       kbHome },
{ Qt::Key_End,        kbEnd },
{ Qt::Key_Left,       kbLeft },
{ Qt::Key_Up,         kbUp },
{ Qt::Key_Right,      kbRight },
{ Qt::Key_Down,       kbDown },
{ Qt::Key_Prior,      kbPgUp },
{ Qt::Key_Next,       kbPgDn },
{ Qt::Key_Shift,      kbShift | kfModifier },
{ Qt::Key_Control,    kbCtrl | kfModifier },
{ Qt::Key_Meta,       kbAlt | kfModifier },
{ Qt::Key_Alt,        kbAlt | kfModifier },
{ Qt::Key_CapsLock,   kbCapsLock | kfModifier },
{ Qt::Key_NumLock,    kbNumLock | kfModifier },
{ Qt::Key_ScrollLock, kbScrollLock | kfModifier },
{ Qt::Key_F1,         kbF1 },
{ Qt::Key_F2,         kbF2 },
{ Qt::Key_F3,         kbF3 },
{ Qt::Key_F4,         kbF4 },
{ Qt::Key_F5,         kbF5 },
{ Qt::Key_F6,         kbF6 },
{ Qt::Key_F7,         kbF7 },
{ Qt::Key_F8,         kbF8 },
{ Qt::Key_F9,         kbF9 },
{ Qt::Key_F10,        kbF10 },
{ Qt::Key_F11,        kbF11 },
{ Qt::Key_F12,        kbF12 },
};

#endif
